from typing import Any, Mapping, Sequence

QUOTE_SAFE_CHAR: str
QUOTE_SAFE_MAPS: Mapping[str, Mapping[Any, str]]
HEX_TO_CHAR_MAP: Mapping[bytes, bytes]

def quote(string_value: str, safe: str = ...) -> str: ...
def quote_plus(string_value: str, safe: str = ...) -> str: ...
def unquote(string_value: str, strict: bool = ...) -> str: ...
def unquote_plus(string_value: str, strict: bool = ...) -> str: ...
def url_encode(
    attributes_map: Mapping[str, str] | None = ...,
    attributes_list: Sequence[tuple[str, str]] | None = ...,
    plus_encoding: bool = ...,
) -> str: ...



from logging import Handler, Logger, Formatter as BaseFormatter
from typing import Mapping

CRITICAL: int
ERROR: int
WARNING: int
INFO: int
DEBUG: int
NOTSET: int
WARN = ...
_levelNames: Mapping[str | int, str | int]
_levelValues: Mapping[str, int]

def getLogger(name: str) -> Logger: ...
def getLevelName(level: str | int) -> str | int: ...
def getLevelInt(levelName: str) -> int: ...

class DummyLogger:
    def __init__(self, name: str, level: int = ...): ...
    def setLevel(self, level: int): ...
    def debug(self, msg: str, *args, **kwargs): ...
    def info(self, msg: str, *args, **kwargs): ...
    def warning(self, msg: str, *args, **kwargs): ...
    def error(self, msg: str, *args, **kwargs): ...
    def exception(self, msg: str, *args): ...
    def critical(self, msg: str, *args, **kwargs): ...
    def addHandler(self, hdlr: Handler): ...
    def removeHandler(self, hdlr: Handler): ...

class StreamHandler:
    formatter: BaseFormatter
    def setFormatter(self, fmt: BaseFormatter): ...

class Formatter:
    def __init__(self, format: str): ...



from typing import Sequence

COEFFICIENT_VALUES: Sequence[int]

def calculate_tax_number_control_value(tax_number: int) -> int: ...
def calculate_id_number_control_value(id_number: int) -> int: ...
def _calculate_control_value(number: int) -> int: ...



from typing import Mapping, Sequence

COUNTRIES: Sequence[str]
COUNTRIES_ISO: Mapping[str, tuple[str, str, str]]

def country_get(
    name: str, relaxed: bool = ...
) -> tuple[str, str, str] | tuple[None, None, None]: ...



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import colony


class CryptTest(colony.ColonyTestCase):
    """
    Class that tests the various functions related
    with the cryptography utilities in Colony.
    """

    def test_md5_crypt(self):
        """
        Tests the MD5 crypt function using some simple values.
        """

        result = colony.md5_crypt("password", "salt")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "$1$salt$qJH7.N4xYta3aEG/dfqo/0")

        result = colony.md5_crypt("password", "01234567")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "$1$01234567$b5lh2mHyD2PdJjFfALlEz1")

    def test_md5_crypt_unicode(self):
        """
        Tests the MD5 crypt function using some simple values
        encoded in unicode.
        """

        result = colony.md5_crypt(colony.legacy.u("password"), colony.legacy.u("salt"))
        self.assertEqual(type(result), colony.legacy.UNICODE)
        self.assertEqual(result, "$1$salt$qJH7.N4xYta3aEG/dfqo/0")

        result = colony.md5_crypt(
            colony.legacy.u("password"), colony.legacy.u("01234567")
        )
        self.assertEqual(type(result), colony.legacy.UNICODE)
        self.assertEqual(result, "$1$01234567$b5lh2mHyD2PdJjFfALlEz1")

        result = colony.md5_crypt(colony.legacy.u("密码"), colony.legacy.u("01234567"))
        self.assertEqual(type(result), colony.legacy.UNICODE)
        self.assertEqual(result, "$1$01234567$MUE6EDF7dbbvoFo3c.Oj1.")



from signal import Signals
from typing import Sequence

WINDOWS_KILL_COMMAND: str
NT_PLATFORM_VALUE: str
DOS_PLATFORM_VALUE: str
WINDOWS_PLATFORMS_VALUE: Sequence[str]

def kill_process(pid: int, signal: Signals | None = ...): ...
def _kill_process_windows(pid: int): ...
def _kill_process_unix(pid: int, _signal: Signals | None = ...): ...



from threading import Condition, Thread
from typing import Any, Callable, Mapping, Sequence

Callback = Callable[[], Any]
CallableList = Sequence[tuple[Callback, int]]
ExceptionHandler = Callable[[Callback, Exception], None]

SCHEDULING_MAX: float

class Scheduler(Thread):
    running_flag: bool
    continue_flag: bool
    busy_flag: bool
    timestamp_queue: Sequence[float]
    timestamp_map: Mapping[float, CallableList]
    tasks: set[int]
    waits: set[int]
    condition: Condition
    waits_condition: Condition
    exception_handler: ExceptionHandler
    _counter: int
    daemon: bool

    def __init__(self): ...
    def run(self): ...
    def start_scheduler(self): ...
    def stop_scheduler(self): ...
    def reset_scheduler(self, notify: bool = ...): ...
    def add_callable(
        self, callable: Callback, timestamp: float | None = ..., verify: bool = ...
    ) -> int: ...
    def wait_callable(self, identifier: int): ...
    def set_exception_handler(self, exception_handler: ExceptionHandler): ...
    def is_busy(self) -> bool: ...
    def is_running(self, pedantic: bool = ...) -> bool: ...
    def _handle_callables(self, callable_list: CallableList): ...



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import config
from . import decorators
from . import exceptions
from . import information
from . import legacy
from . import loggers
from . import system
from . import test
from . import util

from .config import conf, conf_prefix, conf_suffix, conf_s, conf_r, conf_d, conf_ctx
from .decorators import (
    load_plugin,
    plugin_meta_information,
    load_allowed,
    load_allowed_capability,
    unload_allowed,
    unload_allowed_capability,
    inject_dependencies,
    plugin_inject,
    event_handler,
    event_handler_method,
    set_configuration_property,
    set_configuration_property_method,
    unset_configuration_property,
    unset_configuration_property_method,
    plugin_call,
    create_load_plugin_interceptor,
)
from .exceptions import (
    ColonyException,
    OperationalError,
    AssertionError,
    PluginSystemException,
    PluginClassNotAvailable,
    InvalidCommand,
    InvalidArgument,
    SecurityError,
    OperationNotComplete,
    OperationRestart,
)
from .information import (
    VERSION,
    RELEASE,
    BUILD,
    RELEASE_DATE,
    RELEASE_DATE_TIME,
    ENVIRONMENT_VERSION,
    ENVIRONMENT,
    DEFAULT_ENCODING,
    DATE_FORMAT,
    DATE_TIME_FORMAT,
    INFORMATION_PATH,
)
from .loggers import BroadcastHandler, MemoryHandler, LogstashHandler
from .system import (
    System,
    Plugin,
    PluginManagerPlugin,
    PluginManager,
    Dependency,
    PluginDependency,
    PackageDependency,
    Condition,
    OperativeSystemCondition,
    Capability,
    Event,
    PluginThread,
    PluginEventThread,
)
from .test import Test
from .util import (
    CPYTHON_ENVIRONMENT,
    JYTHON_ENVIRONMENT,
    IRON_PYTHON_ENVIRONMENT,
    WINDOWS_OS,
    MAC_OS,
    UNIX_OS,
    OTHER_OS,
    UID_PRECISION,
    WaitInput,
    QueueEvent,
    Plugins,
    module_import,
    resolve_manager,
    ensure_tree,
    is_master,
    get_environment,
    get_operative_system,
    get_timestamp_uid,
)



from typing import Any, Mapping

class DataCacheMap:
    data_map: Mapping[str, tuple[Any, float]]

    def __init__(self): ...
    def get(self, name: str, timestamp: float | None = None) -> Any: ...
    def add(self, name: str, data: Any, timestamp: float): ...
    def remove(self, name: str): ...



from os import PathLike
from typing import Sequence

BUFFER_SIZE: int
LONG_PATH_PREFIX: str
CURRENT_DIRECTORY: str
PARENT_DIRECTORY: str
SEPARATOR: str
NT_PLATFORM_VALUE: str
CE_PLATFORM_VALUE: str
DOS_PLATFORM_VALUE: str
WINDOWS_PLATFORMS_VALUE: Sequence[str]

def normalize_path(path: PathLike[str]) -> PathLike[str]: ...
def align_path(path: PathLike[str]) -> PathLike[str]: ...
def copy_directory(
    source_path: PathLike[str],
    target_path: PathLike[str],
    replace_files: bool = ...,
    copy_hidden: bool = ...,
) -> None: ...
def copy_link(
    source_path: PathLike[str], target_path: PathLike[str], replace_file: bool = ...
): ...
def copy_file(
    source_path: PathLike[str], target_path: PathLike[str], replace_file: bool = ...
): ...
def remove_directory(directory_path: PathLike[str]) -> None: ...
def link(
    target_path: PathLike[str],
    link_path: PathLike[str],
    link_name: bool = ...,
    replace: bool = ...,
): ...
def link_copy(target_path: PathLike[str], link_path: PathLike[str]): ...
def ensure_file_path(file_path: PathLike[str], default_file_path: PathLike[str]): ...
def is_parent_path(path: PathLike[str], parent_path: PathLike[str]) -> bool: ...
def _relative_path_windows(
    path: PathLike[str], start_path: PathLike[str] = ...
) -> PathLike[str]: ...
def _relative_path_posix(
    path: PathLike[str], start: PathLike[str] = ...
) -> PathLike[str]: ...
def _abspath_split(
    path: PathLike[str],
) -> tuple[bool, PathLike[str], Sequence[PathLike[str]]]: ...



from typing import Any, Callable

DEFAULT_NUMBER_RETRIES: int
DEFAULT_RETRY_SLEEP: int

def execute_retries(
    callable: Callable[[], Any], number_retries: int = ..., retry_sleep: float = ...
) -> Any: ...
def call_safe(callable: Callable, *args, **kwargs) -> Any: ...



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

CRITICAL = 50
""" Critical logging level """

ERROR = 40
""" Error logging level """

WARNING = 30
""" Warning logging level """

INFO = 20
""" Info logging level """

DEBUG = 10
""" Debug logging level """

NOTSET = 0
""" Not set logging level """

WARN = WARNING
""" Alias to WARNING log level """

_levelNames = {
    CRITICAL: "CRITICAL",
    ERROR: "ERROR",
    WARNING: "WARNING",
    INFO: "INFO",
    DEBUG: "DEBUG",
    NOTSET: "NOTSET",
    "CRITICAL": CRITICAL,
    "ERROR": ERROR,
    "WARN": WARNING,
    "WARNING": WARNING,
    "INFO": INFO,
    "DEBUG": DEBUG,
    "NOTSET": NOTSET,
}
""" The map relating the log levels with the textual
representation and vice-versa """

_levelValues = {
    "CRITICAL": CRITICAL,
    "ERROR": ERROR,
    "WARN": WARNING,
    "WARNING": WARNING,
    "INFO": INFO,
    "DEBUG": DEBUG,
    "NOTSET": NOTSET,
}
""" Map tha associated the textual representation of the
log level with the integer value """


def getLogger(name):
    """
    Returns the dummy logger for the given name.

    :type name: String
    :param name: The name of the logger to retrieve.
    :rtype: DummyLogger
    :return: The dummy logger for the fiven name.
    """

    return DummyLogger("dummy")


def getLevelName(level):
    """
    Returns the textual representation of logging level integer
    value provided as argument.

    :type level: int
    :param level: The logging level to retrieve the textual representation.
    :rtype: String
    :return: The textual representation of logging level.
    """

    return _levelNames.get(level, ("Level %s" % level))


def getLevelInt(levelName):
    """
    Converts the textual representation of the logging level into
    the integer value.

    :type levelName: String
    :param levelName: The textual representation of the logging level.
    :rtype: int
    :return: The integer value of the logging level.
    """

    return _levelValues.get(levelName, NOTSET)


class DummyLogger(object):
    """
    The dummy logger class.
    """

    def __init__(self, name, level=NOTSET):
        """
        Constructor of the class.

        :type name: String
        :param name: The name of the logger.
        :type level: int
        :param level: The logging level for the new logger.
        """

        pass

    def setLevel(self, level):
        """
        Sets the level of the logger.

        :type level: int
        :param level: The level of the logger.
        """

        pass

    def debug(self, msg, *args, **kwargs):
        """
        Prints a debug message to the logger.

        :type msg: String
        :param msg: The message to print.
        """

        pass

    def info(self, msg, *args, **kwargs):
        """
        Prints an info message to the logger.

        :type msg: String
        :param msg: The message to print.
        """

        pass

    def warning(self, msg, *args, **kwargs):
        """
        Prints a warning message to the logger.

        :type msg: String
        :param msg: The message to print.
        """

        pass

    def error(self, msg, *args, **kwargs):
        """
        Prints an error message to the logger.

        :type msg: String
        :param msg: The message to print.
        """

        pass

    def exception(self, msg, *args):
        """
        Prints an exception message to the logger.

        :type msg: String
        :param msg: The message to print.
        """

        pass

    def critical(self, msg, *args, **kwargs):
        """
        Prints a critical message to the logger.

        :type msg: String
        :param msg: The message to print.
        """

        pass

    def addHandler(self, hdlr):
        """
        Adds an handler to the logger.

        :type hdlr: Handler
        :param hdlr: The handler to add to the logger.
        """

        pass

    def removeHandler(self, hdlr):
        """
        Removes an handler from the logger.

        :type hdlr: Handler
        :param hdlr: The handler to remove from the logger.
        """

        pass


class StreamHandler(object):
    """
    The stream handler class.
    """

    def setFormatter(self, fmt):
        """
        Sets the formatter for this handler.
        """

        self.formatter = fmt


class Formatter(object):
    """
    The formatter class.
    """

    def __init__(self, format):
        """
        Constructor of the class.

        :type format: String
        :param format: The formatter format.
        """

        pass



from socket import AddressFamily, SocketKind
from typing import Any, Sequence

LOCAL_EXTENSION: str
LOOPBACK_IP4_ADDRESS: str
LOOPBACK_IP6_ADDRESS: str
ALL_IP4_ADDRESS: str
ALL_IP6_ADDRESS: str
DEFAULT_IP4_HOST: str
DEFAULT_IP6_HOST: str
DEFAULT_PORT: int

def get_hostname() -> str: ...
def get_hostname_local() -> str: ...
def get_address_ip4() -> str: ...
def get_address_ip4_force(host: str = ..., port: int = ...) -> str: ...
def get_address_ip4_all() -> str: ...
def get_address_ip6() -> str: ...
def get_address_ip6_force(host: str = ..., port: int = ...) -> str: ...
def get_address_ip6_all() -> str: ...
def get_addresses_ip4() -> Sequence[str]: ...
def get_addresses_ip6() -> Sequence[str]: ...
def get_addresses_family(
    filter_family: AddressFamily, filter_addresses: Sequence[str] = ...
) -> Sequence[str]: ...
def get_address_tuples() -> (
    Sequence[tuple[AddressFamily, SocketKind, int, str, Any]]
): ...
def ip4_address_from_network(ip4_address_network: str) -> str: ...
def ip4_address_to_network(ip4_address: str) -> str: ...
def ip6_address_from_network(ip6_address_network: str) -> str: ...
def ip6_address_to_network(ip6_address: str) -> str: ...



from typing import Any, Mapping, Sequence, Type

TOPPER_VALUE: str
LIST_TYPES: Sequence[Type]
INVALID_ATTRIBUTE_NAMES: Sequence[str]
VALID_ATTRIBUTE_TYPES: Sequence[Type]

def object_attribute_names(instance: Any) -> Sequence[str]: ...
def object_attribute_values(
    instance: Any,
    valid_attribute_names: Sequence[str] | None = ...,
    strict: bool = ...,
) -> Sequence[Any]: ...
def object_flatten(
    instance: Any, flattening_map: Mapping[Any, Any]
) -> Sequence[Any]: ...
def object_print_list(instances_list: Sequence[Any]): ...
def object_print(instance: Any): ...
def _object_flatten(
    instances_list: Sequence[Any], flattening_map: Mapping[Any, Any]
) -> Sequence[Any]: ...



from os import PathLike
from re import Pattern
from typing import Literal, Sequence

HashType = Literal["md5", "sha1", "sha256"]

HASH_VALUE: str
VALUE_VALUE: str
PLAIN_VALUE: str
MD5_VALUE: str
SHA1_VALUE: str
SHA256_VALUE: str
MD5_CRYPT_SEPARATOR: str
DEFAULT_MD5_CRYPT_MAGIC: str
DEFAULT_HASH_SET: HashType
INTEGER_TO_ASCII_64: str
PASSWORD_VALUE_REGEX_VALUE: str
NUMBER_REGEX_VALUE: str
LETTER_LOWER_REGEX_VALUE: str
LETTER_UPPER_REGEX_VALUE: str
SPECIAL_CHARACTER_REGEX_VALUE: str
PASSWORD_VALUE_REGEX: Pattern[str]
NUMBER_REGEX: Pattern[str]
LETTER_LOWER_REGEX: Pattern[str]
LETTER_UPPER_REGEX: Pattern[str]
SPECIAL_CHARACTER_REGEX: Pattern[str]

def password_crypt(
    password: str, salt: str = ..., hash_method: HashType = ...
) -> str: ...
def password_match(password_hash: str, password: str, salt: str = ...) -> bool: ...
def password_strength(password: str) -> int: ...
def md5_crypt(password: str, salt: str, magic: str = ...) -> str: ...
def generate_hash_digest_map(
    file_path: PathLike[str], hash_set: Sequence[str] = ...
): ...



from typing import Any, Callable, Generator, Mapping, Sequence, Type, TypeVar

T = TypeVar("T")
V = TypeVar("V")

def map_clean(map: Mapping[Any, Any]): ...
def map_get(map: Mapping[Any, Any], keys: Sequence[Any] = []): ...
def map_copy(source_map: Mapping[Any, Any], destiny_map: Mapping[Any, Any]): ...
def map_copy_deep(source_map: Mapping[Any, Any], destiny_map: Mapping[Any, Any]): ...
def map_duplicate(item: Any) -> Any: ...
def map_remove(removal_map: Mapping[Any, Any], destiny_map: Mapping[Any, Any]): ...
def map_extend(
    base_map: Mapping[Any, Any],
    extension_map: Mapping[Any, Any],
    override: bool = ...,
    recursive: bool = ...,
    copy_base_map: bool = ...,
): ...
def map_flatten(map: Mapping[str, Any]) -> Mapping[str, Any]: ...
def map_check_parameters(
    map: Mapping[T, Any], parameters_list: Sequence[T], exception: Type[Exception] = ...
): ...
def map_get_value_cast(
    map: Mapping[T, Any], key: T, cast_type: Type = ..., default_value: Any | None = ...
) -> Any: ...
def map_get_values(map: Mapping[T, Any], key: T) -> Sequence[Any]: ...
def map_output(
    map: Mapping[Any, Any],
    output_method: Callable[[bytes | str], Any] = ...,
    indentation: str = ...,
): ...
def map_normalize(item: T, operation: Callable[[T], Any] | None = ...) -> Any: ...
def _map_flatten_pairs(map: Mapping[T, V]) -> Generator[tuple[T, V], None, None]: ...
def _map_reduce(value: Any) -> Any: ...



from re import Pattern

UNDERSCORE_FIRST_VALUE: str
UNDERSCORE_FIRST_REGEX: Pattern[str]
UNDERSCORE_SECOND_VALUE: str
UNDERSCORE_SECOND_REGEX: Pattern[str]

def xor_string_value(first_string: str, second_string: str) -> str: ...
def to_underscore(string_value: str) -> str: ...
def to_camelcase(string_value: str) -> str: ...
def pluralize(string_value: str) -> str: ...
def capitalize_all(string_value: str) -> str: ...
def join(first_value: str, second_value: str, join_value: str = ...) -> str: ...



from typing import Any, Iterator, TypeVar

T = TypeVar("T")

class LazyClass:
    def __repr__(self) -> str: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __add__(self, other: Any) -> Any: ...
    def __radd__(self, other: Any) -> Any: ...
    def __nonzero__(self) -> bool: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[Any]: ...

class LazyIteratorClass[T]:
    def __iter__(self) -> Iterator: ...
    def __next__(self) -> T: ...
    def next(self) -> T: ...

def is_lazy(value: Any) -> bool: ...

Lazy: LazyClass
LazyIterator: LazyIteratorClass



from typing import Any, Callable, Mapping, Sequence

HandlersMap = Mapping[str, Sequence[Callable]]

COUNTER: int
MESSAGE_VALUE: str
ACTION_VALUE: str
PROGRESS_VALUE: str
GLOBAL_HANDLERS_MAP: HandlersMap
KAFKA_PRODUCERS: Mapping[str, Any]
_KAFKA_CONFIG: Mapping[str, Any]

def unique() -> int: ...
def notify(
    operation_name: str,
    handlers_map: Mapping[str, Sequence[Callable]] | None = ...,
    *arguments,
    **named_arguments
) -> Any: ...
def message(
    handlers_map: HandlersMap | None = ..., *arguments, **named_arguments
) -> Any: ...
def action(
    handlers_map: HandlersMap | None = ..., *arguments, **named_arguments
) -> Any: ...
def progress(
    handlers_map: HandlersMap | None = ..., *arguments, **named_arguments
) -> Any: ...
def register_g(operation_name: str, handler: Callable): ...
def unregister_g(
    operation_name: str,
    handler: Callable | None = ...,
): ...
def notify_g(operation_name: str, *arguments, **named_arguments) -> Any: ...
def notify_b(operation_name: str, *arguments, **named_arguments): ...
def notify_kafka(operation_name: str, *arguments, **named_arguments): ...
def kafka_config() -> Mapping[str, Any]: ...
def _kafka_producer() -> Any: ...



BLOCK_SIZE: int

class AesCipher:
    key: bytes | None
    block_size = BLOCK_SIZE

    def __init__(self, key: bytes | None = None, block_size: int = ...): ...
    def encrypt(self, raw: bytes) -> bytes: ...
    def decrypt(self, encoded: bytes) -> bytes: ...
    def pad(self, value: bytes) -> bytes: ...
    def unpad(self, value: bytes) -> bytes: ...
    def get_key(self) -> bytes: ...
    def get_block_size(self) -> int: ...



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import aes_util
from . import bank_util
from . import barcode_util
from . import cache_util
from . import call_util
from . import control_util
from . import country_util
from . import crypt_util
from . import encode_util
from . import file_util
from . import gtin_util
from . import host_util
from . import import_util
from . import lazy_util
from . import list_util
from . import logging_util
from . import map_util
from . import math_util
from . import number_util
from . import object_util
from . import observer_util
from . import os_util
from . import path_util
from . import protection_util
from . import quote_util
from . import round_util
from . import scheduling_util
from . import size_util
from . import stack_util
from . import string_buffer_util
from . import string_util
from . import structures_util
from . import test_util
from . import time_util
from . import update_thread_util
from . import verify_util
from . import version_util
from . import visitor_util
from . import xml_util

from .aes_util import AesCipher
from .barcode_util import encode_2_of_5, encode_code_128, encode_code_39
from .cache_util import DataCacheMap
from .call_util import execute_retries, call_safe
from .control_util import (
    calculate_tax_number_control_value,
    calculate_id_number_control_value,
)
from .country_util import COUNTRIES, country_get
from .crypt_util import (
    password_crypt,
    password_match,
    password_strength,
    md5_crypt,
    generate_hash_digest_map,
)
from .encode_util import encode_two_complement_string, decode_two_complement_string
from .file_util import (
    FileRotator,
    FileContext,
    TransactionContext,
    FileImmediateContext,
    FileTransactionContext,
)
from .host_util import (
    get_hostname,
    get_hostname_local,
    get_address_ip4,
    get_address_ip4_force,
    get_address_ip4_all,
    get_address_ip6,
    get_address_ip6_force,
    get_address_ip6_all,
    get_addresses_ip4,
    get_addresses_ip6,
    get_addresses_family,
    get_address_tuples,
    ip4_address_from_network,
    ip4_address_to_network,
    ip6_address_from_network,
    ip6_address_to_network,
)
from .import_util import reload_import
from .lazy_util import LazyClass, LazyIteratorClass, is_lazy, Lazy, LazyIterator
from .list_util import list_intersect, list_extend, list_no_duplicates
from .logging_util import (
    getLogger,
    getLevelName,
    getLevelInt,
    DummyLogger,
    StreamHandler,
    Formatter,
)
from .map_util import (
    map_clean,
    map_get,
    map_copy,
    map_copy_deep,
    map_duplicate,
    map_remove,
    map_extend,
    map_flatten,
    map_check_parameters,
    map_get_value_cast,
    map_get_values,
    map_output,
    map_normalize,
)
from .math_util import (
    ceil_integer,
    greatest_common_divisor,
    fast_exponentiation,
    item_set_total,
    item_set_percentage,
)
from .number_util import get_number_length, get_digit, to_fixed
from .object_util import (
    object_attribute_names,
    object_attribute_values,
    object_flatten,
    object_print_list,
    object_print,
)
from .observer_util import (
    unique,
    notify,
    message,
    action,
    progress,
    register_g,
    unregister_g,
    notify_g,
    notify_b,
    notify_kafka,
    kafka_config,
)
from .os_util import kill_process
from .path_util import (
    SEPARATOR,
    normalize_path,
    align_path,
    copy_directory,
    copy_link,
    copy_file,
    remove_directory,
    link,
    link_copy,
    ensure_file_path,
    is_parent_path,
    relative_path,
)
from .protection_util import public, Protected
from .quote_util import quote, quote_plus, unquote, unquote_plus, url_encode
from .round_util import roundi, rounds, roundt, round_apply, round_unapply, round_is_new
from .scheduling_util import SCHEDULING_MAX, Scheduler
from .size_util import size_round_unit
from .stack_util import get_instance_module_directory, get_call_module_directory
from .string_buffer_util import StringBuffer
from .string_util import (
    xor_string_value,
    to_underscore,
    to_camelcase,
    pluralize,
    capitalize_all,
    join,
)
from .structures_util import (
    Decimal,
    JournaledList,
    OrderedMap,
    OrderedMapIterator,
    MultipleValueMap,
    FormatTuple,
    FileReference,
    is_dictionary,
)
from .test_util import ColonyTestCase
from .time_util import (
    SIMPLE_VALUE,
    BASIC_VALUE,
    EXTENDED_VALUE,
    EXTENDED_SIMPLE_VALUE,
    MINIMIZE_MULTIPLE,
    MINIMIZE_UNIQUE,
    format_seconds_smart,
    format_seconds,
    timestamp_datetime,
)
from .update_thread_util import UpdateThread
from .verify_util import (
    verify,
    verify_equal,
    verify_not_equal,
    verify_type,
    verify_many,
)
from .version_util import version_cmp, version_is_concrete
from .visitor_util import visit, dispatch_visit
from .xml_util import xml_to_dict, dict_to_xml

from .bank_util import calculate_control_value as calculate_control_value_bank
from .gtin_util import calculate_control_value as calculate_control_value_gtin

from .import_util import __import__



from os import PathLike
from typing import Self, Sequence, Type, TypeVar

T = TypeVar("T", bound=bytes | str)

class StringBuffer[T]:
    softspace: int
    closed: bool
    string_list: Sequence[T]
    current_value: T
    btype: Type
    dirty: bool
    current_position: int
    current_size: int
    fast: bool

    def __init__(self, fast: bool = ..., btype: Type | None = ...): ...
    def read(self, size: int | None = None) -> T: ...
    def write(self, string_value: T) -> int: ...
    def close(self): ...
    def flush(self): ...
    def reset(self): ...
    def seek(self, offset: int, whence: int = ...): ...
    def eof(self) -> bool: ...
    def next(self): ...
    def tell(self) -> int: ...
    def truncate(self): ...
    def readline(self, size: int | None = None) -> T: ...
    def readlines(self, sizehint: int | None = None) -> Sequence[T]: ...
    def writelines(self, lines: Sequence[T]): ...
    def isatty(self) -> bool: ...
    def getvalue(self) -> T: ...
    def get_value(self) -> T: ...
    def is_empty(self) -> bool: ...
    def regenerate(self): ...
    def duplicate(self) -> Self: ...
    def rollback_last(self, item_count: int = ...): ...
    def get_last(self, index: int = ...) -> T: ...
    def output_file(self, path: PathLike[str]): ...
    def _write_fast(self, string_value: T): ...
    def _write_slow(self, string_value: T): ...
    def _regenerate(self): ...
    def _base_type(self) -> Type: ...



FLOAT_PRECISION: int
DELTA: float

def roundi(value: float, places: int = ..., precise: bool = ...) -> float: ...
def rounds(value: float, places: int = ...) -> float: ...
def roundt(value: float, places: int = ...) -> float: ...
def round_apply(force: bool = ...): ...
def round_unapply(force: bool = ...): ...
def round_is_new() -> bool: ...
def _round_t(value: float, places: int = ...) -> float: ...
def _delta(value: float) -> float: ...



from threading import Thread
from _typeshed import Incomplete

GLOBAL_CONFIG: Incomplete
CPYTHON_ENVIRONMENT: Incomplete
JYTHON_ENVIRONMENT: Incomplete
IRON_PYTHON_ENVIRONMENT: Incomplete
DEFAULT_LOGGER: str
DEFAULT_LOGGING_LEVEL: Incomplete
DEFAULT_LOGGING_FORMAT: str
DEFAULT_LOGGING_FILE_NAME_PREFIX: str
DEFAULT_LOGGING_FILE_NAME_SEPARATOR: str
DEFAULT_LOGGING_FILE_NAME_EXTENSION: str
DEFAULT_LOGGING_ERR_FILE_NAME_EXTENSION: str
DEFAULT_LOGGING_FILE_MODE: str
DEFAULT_LOGGING_FILE_SIZE: int
DEFAULT_LOGGING_FILE_BACKUP_COUNT: int
DEFAULT_CONTAINERS_PATH: str
DEFAULT_LIBRARIES_PATH: str
DEFAULT_TEMPORARY_PATH: str
DEFAULT_VARIABLE_PATH: str
DEFAULT_PLUGIN_PATH: str
DEFAULT_CONFIGURATION_PATH: str
DEFAULT_PLUGIN_PATHS_FILE_PATH: str
DEFAULT_WORKSPACE_PATH: str
DEFAULT_UNLOAD_SYSTEM_TIMEOUT: float
EAGER_LOADING_TYPE: str
LAZY_LOADING_TYPE: str
PLUGIN_MANAGER_EXTENSION_TYPE: str
MAIN_TYPE: str
STARTUP_TYPE: str
THREAD_TYPE: str
FULL_LOAD_TYPE: str
DEPENDENCY_TYPE: str
ALLOWED_TYPE: str
SINGLETON_DIFFUSION_SCOPE: int
SAME_DIFFUSION_SCOPE: int
NEW_DIFFUSION_SCOPE: int
FILE_REMOVED_TYPE: str
SPECIAL_VALUE_REGEX_VALUE: str
SPECIAL_VALUE_REGEX: Incomplete
ALIAS_MAP: Incomplete

class System:
    plugin: Plugin | None

    def __init__(self, plugin) -> None: ...
    def get_manager(self): ...
    def debug(self, *args, **kwargs): ...
    def info(self, *args, **kwargs): ...
    def warning(self, *args, **kwargs): ...
    def error(self, *args, **kwargs): ...
    def critical(self, *args, **kwargs): ...

class Plugin:
    id: Incomplete
    name: Incomplete
    description: Incomplete
    version: Incomplete
    author: Incomplete
    loading_type = EAGER_LOADING_TYPE
    platforms: Incomplete
    attributes: Incomplete
    capabilities: Incomplete
    capabilities_allowed: Incomplete
    dependencies: Incomplete
    events_fired: Incomplete
    events_handled: Incomplete
    main_modules: Incomplete
    valid: bool
    logger: Incomplete
    timestamp: Incomplete
    dependencies_loaded: Incomplete
    allowed_loaded_capability: Incomplete
    event_plugins_fired_loaded_map: Incomplete
    event_plugins_registered_loaded_map: Incomplete
    event_plugin_manager_registered_loaded_list: Incomplete
    configuration_map: Incomplete
    loaded: bool
    lazy_loaded: bool
    error_state: bool
    exception: Incomplete
    ready_semaphore: Incomplete
    ready_semaphore_lock: Incomplete
    ready_semaphore_release_count: int
    original_id: Incomplete
    diffusion_scope_id: Incomplete
    manager: Incomplete

    def __init__(self, manager: Incomplete | None = ...): ...
    def __repr__(self) -> str: ...
    def load_plugin(self) -> None: ...
    def lazy_load_plugin(self) -> None: ...
    def end_load_plugin(self) -> None: ...
    def unload_plugin(self) -> None: ...
    def end_unload_plugin(self) -> None: ...
    def load_allowed(self, plugin, capability) -> None: ...
    def unload_allowed(self, plugin, capability) -> None: ...
    def dependency_injected(self, plugin) -> None: ...
    def init_complete(self) -> None: ...
    def register_all_handled_events_plugin(self, plugin) -> None: ...
    def unregister_all_handled_events_plugin(self, plugin) -> None: ...
    def register_all_plugin_manager_events(self) -> None: ...
    def unregister_all_plugin_manager_events(self) -> None: ...
    def register_for_plugin_event(self, plugin, event_name) -> None: ...
    def unregister_for_plugin_event(self, plugin, event_name) -> None: ...
    def register_for_plugin_manager_event(self, event_name) -> None: ...
    def unregister_for_plugin_manager_event(self, event_name) -> None: ...
    def unregister_all_for_plugin_event(self, event_name) -> None: ...
    def unregister_all_for_plugin(self) -> None: ...
    def register_plugin_event(self, plugin, event_name) -> None: ...
    def unregister_plugin_event(self, plugin, event_name) -> None: ...
    def notify_handlers(self, event_name, event_args) -> None: ...
    def generate_event(self, event_name, event_args) -> None: ...
    def event_handler(self, event_name, *event_args) -> None: ...
    def reload_main_modules(self): ...
    def get_configuration_property(self, property_name): ...
    def set_configuration_property(self, property_name, property) -> None: ...
    def unset_configuration_property(self, property_name) -> None: ...
    def ensure(self) -> None: ...
    def is_loaded(self): ...
    def is_lazy_loaded(self): ...
    def is_loaded_or_lazy_loaded(self): ...
    def is_replica(self): ...
    def get_attribute(self, attribute_name, default: Incomplete | None = None): ...
    def has_capability(self, capability): ...
    def contains_metadata(self): ...
    def contains_metadata_key(self, metadata_key): ...
    def get_metadata(self): ...
    def get_metadata_key(self, metadata_key): ...
    def treat_exception(self, exception) -> None: ...
    def acquire_ready_semaphore(self) -> None: ...
    def release_ready_semaphore(self) -> None: ...
    def ready_semaphore_status(self): ...
    def get_all_plugin_dependencies(self): ...
    def get_all_package_dependencies(self): ...
    def get_tuple(self): ...
    def get_author_name(self): ...
    def get_uptime(self): ...
    def log_stack_trace(self, level=...) -> None: ...
    def debug(self, message) -> None: ...
    def info(self, message) -> None: ...
    def warning(self, message) -> None: ...
    def error(self, message) -> None: ...
    def critical(self, message) -> None: ...
    def format_logger_message(self, message): ...
    def _get_capabilities_allowed_names(self): ...

class PluginManagerPlugin(Plugin):
    valid: bool
    def __init__(self, manager: Incomplete | None = None) -> None: ...

class PluginManager:
    uid: Incomplete
    logger: Incomplete
    logger_handlers: Incomplete
    platform: Incomplete
    condition: Incomplete
    init_complete: bool
    blacklist: Incomplete
    blacktest: Incomplete
    init_complete_handlers: Incomplete
    main_loop_active: bool
    auto_unload: bool
    allow_threads: bool
    install_signals: bool
    layout_mode: str
    run_mode: str
    container: str
    daemon_pid: Incomplete
    daemon_file_path: Incomplete
    prefix_paths: Incomplete
    configuration_path = DEFAULT_CONFIGURATION_PATH
    workspace_path = DEFAULT_WORKSPACE_PATH
    timestamp: int
    plugin_manager_plugins_loaded: bool
    plugins: Incomplete
    retrieve_lock: Incomplete
    current_id: int
    replica_id: int
    diffusion_scope_id: int
    return_code: int
    event_queue: Incomplete
    manager_path: Incomplete
    logger_path: Incomplete
    library_paths: Incomplete
    plugin_paths: Incomplete
    kill_system_timer: Incomplete
    referred_modules: Incomplete
    loaded_plugins: Incomplete
    loaded_plugins_map: Incomplete
    loaded_plugins_id_map: Incomplete
    id_loaded_plugins_map: Incomplete
    loaded_plugins_descriptions: Incomplete
    plugin_classes: Incomplete
    plugin_classes_map: Incomplete
    plugin_instances: Incomplete
    plugin_instances_map: Incomplete
    plugin_names_map: Incomplete
    plugin_dirs_map: Incomplete
    capabilities_plugin_instances_map: Incomplete
    capabilities_sub_capabilities_map: Incomplete
    plugin_threads: Incomplete
    plugin_threads_map: Incomplete
    plugin_dependent_plugins_map: Incomplete
    plugin_allowed_plugins_map: Incomplete
    capabilities_plugins_map: Incomplete
    diffusion_scope_loaded_plugins_map: Incomplete
    deleted_plugin_classes: Incomplete
    event_plugins_fired_loaded_map: Incomplete
    meta_paths: Incomplete
    stop_on_cycle_error: Incomplete
    whitetest: Incomplete
    exec_delay: Incomplete

    def __init__(
        self,
        manager_path: str = "",
        logger_path: str = "log",
        library_paths=[],
        meta_paths=[],
        plugin_paths=[],
        platform=...,
        init_complete_handlers=[],
        stop_on_cycle_error: bool = True,
        loop: bool = True,
        threads: bool = True,
        signals: bool = True,
        layout_mode: str = "default",
        run_mode: str = "default",
        container: str = "default",
        prefix_paths=[],
        daemon_pid: Incomplete | None = None,
        daemon_file_path: Incomplete | None = None,
    ) -> None: ...
    def create_plugin(self, plugin_id, plugin_version): ...
    def _create_plugin(self, plugin_id, plugin_version, diffusion_scope_id): ...
    def generate_replica_id(self): ...
    def generate_diffusion_scope_id(self): ...
    def start_logger(self, log_level=...) -> None: ...
    def load_system(
        self,
        mode: Incomplete | None = None,
        args: Incomplete | None = None,
        callback: Incomplete | None = None,
    ): ...
    def unload_system(self, thread_safe: bool = True) -> None: ...
    def reload_system(self, thread_safe: bool = True) -> None: ...
    def main_loop(self, timeout: float = 1.0) -> None: ...
    def add_event(self, event) -> None: ...
    def expand_workspace_path(self) -> None: ...
    def create_workspace_path(self) -> None: ...
    def update_workspace_path(self) -> None: ...
    def check_standard_input(self) -> None: ...
    def apply_fixes(self) -> None: ...
    def get_all_modules(self, path, suffix: Incomplete | None = None): ...
    def init_plugin_system(self, configuration) -> None: ...
    def set_python_path(self, library_paths, plugin_paths) -> None: ...
    def load_plugins(self, plugins) -> None: ...
    def start_plugin_manager_plugins(self) -> None: ...
    def start_plugins(self) -> None: ...
    def start_plugin(self, plugin, use_path: bool = True) -> None: ...
    def stop_plugin_complete_by_id(self, plugin_id) -> None: ...
    def stop_module(self, module) -> None: ...
    def stop_plugin(self, plugin) -> None: ...
    def add_plugin_path(self, plugin_path, persist: bool = False) -> None: ...
    def remove_plugin_path(self, plugin_path) -> None: ...
    def persist_plugin_path(self, plugin_path) -> None: ...
    def get_all_plugin_classes(self, base_plugin_class=...): ...
    def get_plugin_sub_classes(self, plugin, plugin_classes) -> None: ...
    def register_plugin_capabilities(self, plugin) -> None: ...
    def unregister_plugin_capabilities(self, plugin) -> None: ...
    def load_plugin_manager_plugins(self) -> None: ...
    def load_startup_plugins(self) -> None: ...
    def load_main_plugins(self) -> None: ...
    def install_signal_handlers(self) -> None: ...
    def notify_load_complete_loaded_plugins(self) -> None: ...
    def notify_load_complete_handlers(self) -> None: ...
    def notify_daemon_file(self) -> None: ...
    def exec_mode(self, mode, args: Incomplete | None = None) -> None: ...
    def run_dry(self, args=[]) -> None: ...
    def run_test(self, verbosity: int = 2, raise_e: bool = True, args=[]): ...
    def __load_plugin(
        self,
        plugin,
        type: Incomplete | None = None,
        loading_type: Incomplete | None = None,
    ): ...
    def _load_plugin(
        self,
        plugin,
        type: Incomplete | None = None,
        loading_type: Incomplete | None = None,
    ): ...
    def _unload_plugin(
        self,
        plugin,
        type: Incomplete | None = None,
        unloading_type: Incomplete | None = None,
    ): ...
    def _unload_thread_plugins(self) -> None: ...
    def test_plugin_load(self, plugin): ...
    def test_dependencies(self, plugin): ...
    def test_platform(self, plugin): ...
    def test_threads(self, plugin): ...
    def test_blacklist(self, plugin): ...
    def resolve_capabilities(self, plugin): ...
    def inject_dependencies(self, plugin): ...
    def inject_allowed(self, plugin): ...
    def _inject_allowed(self, plugin, allowed_plugin, capability) -> None: ...
    def inject_all_allowed(self, plugin) -> None: ...
    def set_plugin_instance_diffusion_scope_loaded_plugins_map(
        self, diffusion_scope_id, plugin_id, plugin_instance
    ) -> None: ...
    def unset_plugin_instance_diffusion_scope_loaded_plugins_map(
        self, diffusion_scope_id, plugin_id
    ) -> None: ...
    def add_plugin_dependent_plugins_map(
        self, plugin_id, dependency_plugin_instance
    ) -> None: ...
    def get_plugin_dependent_plugins_map(self, plugin_id): ...
    def clear_plugin_dependent_plugins_map(self, plugin_id) -> None: ...
    def add_plugin_allowed_plugins_map(
        self, plugin_id, allowed_plugin_info_list
    ) -> None: ...
    def get_plugin_allowed_plugins_map(self, plugin_id): ...
    def clear_plugin_allowed_plugins_map(self, plugin_id) -> None: ...
    def add_capabilities_plugins_map(self, capability, plugin) -> None: ...
    def get_capabilities_plugins_map(self, capability): ...
    def clear_capabilities_plugins_map(self, capability) -> None: ...
    def clear_capabilities_plugins_map_for_plugin(self, plugin_id) -> None: ...
    def load_plugin(self, plugin_id, type: Incomplete | None = None): ...
    def unload_plugin(self, plugin_id, type: Incomplete | None = None): ...
    def get_all_plugins(self): ...
    def get_all_loaded_plugins(self): ...
    def ensure(self, plugin) -> None: ...
    def assert_plugin(self, plugin): ...
    def get_plugin(self, plugin_id, plugin_version: Incomplete | None = None): ...
    def _get_plugin(self, plugin_id, plugin_version: Incomplete | None = None): ...
    def get_plugin_by_id(self, plugin_id): ...
    def _get_plugin_by_id(self, plugin_id) -> Plugin | None: ...
    def get_plugin_by_id_and_version(self, plugin_id, plugin_version): ...
    def _get_plugin_by_id_and_version(
        self, plugin_id, plugin_version
    ) -> Plugin | None: ...
    def get_plugins_by_capability(self, capability): ...
    def _get_plugins_by_capability_cache(self, capability): ...
    def _get_plugins_by_capability(self, capability): ...
    def __get_plugins_by_capability(self, capability): ...
    def get_plugins_by_capability_allowed(self, capability_allowed): ...
    def _get_plugins_by_capability_allowed(self, capability_allowed): ...
    def get_plugins_by_event_fired(self, event_fired): ...
    def _get_plugins_by_event_fired(self, event_fired): ...
    def get_plugins_by_event_handled(self, event_handled): ...
    def _get_plugins_by_event_handled(self, event_handled): ...
    def get_plugins_by_dependency(self, plugin_id): ...
    def _get_plugins_by_dependency(self, plugin_id): ...
    def get_plugins_allow_capability(self, capability): ...
    def _get_plugins_allow_capability(self, capability): ...
    def resolve_file_path(
        self, file_path, not_found_valid: bool = False, create_path: bool = False
    ): ...
    def resolve_string_value(self, string_value): ...
    def get_plugin_path_by_id(self, plugin_id): ...
    def get_temporary_plugin_path_by_id(self, plugin_id, extra_path: str = ""): ...
    def get_temporary_plugin_generated_path_by_id(self, plugin_id): ...
    def get_plugin_configuration_paths_by_id(
        self, plugin_id, extra_paths: bool = False
    ): ...
    def get_plugin_configuration_file_by_id(
        self, plugin_id, configuration_file_path
    ): ...
    def get_plugin_module_name_by_id(self, plugin_id): ...
    def get_plugin_by_module_name(self, module): ...
    def get_loaded_plugin_by_module_name(self, module): ...
    def get_plugin_class_by_module_name(self, module): ...
    def register_plugin_manager_event(self, plugin, event_name) -> None: ...
    def unregister_plugin_manager_event(self, plugin, event_name) -> None: ...
    def notify_handlers(self, event_name, event_args) -> None: ...
    def generate_event(self, event_name, event_args) -> None: ...
    def plugin_manager_plugin_execute(self, execution_type, arguments): ...
    def plugin_manager_plugin_execute_conditional(self, execution_type, arguments): ...
    def exists_plugin_manager_plugin_execute_conditional(
        self, execution_type, arguments
    ): ...
    system_information_map: Incomplete
    def generate_system_information_map(self) -> None: ...
    def get_log_handler(self, name): ...
    def log_stack_trace(self, level=...) -> None: ...
    def debug(self, message) -> None: ...
    def info(self, message) -> None: ...
    def warning(self, message) -> None: ...
    def error(self, message) -> None: ...
    def critical(self, message) -> None: ...
    def format_logger_message(self, message): ...
    def print_all_plugins(self) -> None: ...
    def get_prefix_paths(self): ...
    def get_environment_variable(self, environment_variable_name): ...
    def get_configuration_path(self): ...
    def get_meta_paths(self): ...
    def get_workspace_path(self): ...
    def set_workspace_path(self, workspace_path) -> None: ...
    def set_timestamp(self, timestamp: Incomplete | None = None) -> None: ...
    def set_plugin_manager_plugins_loaded(self, value: bool = True) -> None: ...
    def get_plugin_manager_plugins_loaded(self): ...
    def set_init_complete(self, value: bool = True) -> None: ...
    def get_init_complete(self): ...
    def get_manager_path(self): ...
    def get_plugin_paths(self): ...
    def get_main_plugin_path(self): ...
    def get_containers_path(self): ...
    def get_libraries_path(self): ...
    def get_temporary_path(self): ...
    def get_variable_path(self): ...
    def get_plugin_paths_file_path(self): ...
    def get_layout_mode(self): ...
    def get_run_mode(self): ...
    def get_timestamp(self): ...
    def get_version(self): ...
    def get_release(self): ...
    def get_build(self): ...
    def get_release_date(self): ...
    def get_release_date_time(self): ...
    def get_environment(self): ...
    def get_system_information_map(self): ...
    def get_uptime(self): ...
    def is_development(self): ...
    def is_production(self): ...
    def echo(self, value: str = "echo"): ...
    def process_command_manager_path(self, arguments): ...
    def process_command_plugin_path(self, arguments): ...
    def process_command_configuration(self, arguments): ...
    def process_command_environment(self, arguments): ...
    def process_command_prefix(self, arguments): ...
    def _relaunch_system(self) -> None: ...
    def _kill_system_signal_handler(self, signum, frame) -> None: ...
    def _stop_blocking_system_structures(self) -> None: ...
    def _stop_kill_system_timer(self) -> None: ...
    def _kill_system_timeout(self) -> None: ...
    def _handle_system_exception(self, exception) -> None: ...
    def _get_best_plugin_path(self, plugin_path): ...

class Dependency:
    mandatory: bool
    conditions_list: bool

    def __init__(self, mandatory: bool = True, conditions_list=[]) -> None: ...
    def test_dependency(self, manager): ...
    def test_conditions(self): ...
    def get_tuple(self): ...

class PluginDependency(Dependency):
    id: Incomplete
    version: Incomplete
    diffusion_policy = SINGLETON_DIFFUSION_SCOPE

    def __init__(
        self,
        id,
        version: str = "x.x.x",
        diffusion_policy=...,
        mandatory: bool = True,
        conditions_list=[],
    ) -> None: ...
    def __repr__(self) -> str: ...
    def test_dependency(self, manager): ...
    def get_tuple(self): ...
    def get_map(self): ...

class PackageDependency(Dependency):
    name: Incomplete
    import_name: Incomplete
    version: str
    url: Incomplete

    def __init__(
        self,
        name,
        import_name,
        version: str = "1.0.0",
        url: Incomplete | None = None,
        mandatory: bool = True,
        conditions_list=[],
    ) -> None: ...
    def __repr__(self) -> str: ...
    def test_dependency(self, manager): ...
    def get_tuple(self): ...
    def get_map(self): ...

class Condition:
    def __init__(self) -> None: ...
    def test_condition(self): ...

class OperativeSystemCondition(Condition):
    operative_system_name: Incomplete
    def __init__(self, operative_system_name: Incomplete | None = None) -> None: ...
    def test_condition(self): ...

class Capability:
    list_value: Incomplete
    def __init__(self, string_value: Incomplete | None = None) -> None: ...
    def __eq__(self, capability): ...
    def __ne__(self, capability): ...
    def capability_and_super_capabilites(self): ...
    def is_sub_capability(self, capability): ...
    def is_capability_or_sub_capability(self, capability): ...

class Event:
    list_value: Incomplete
    def __init__(self, string_value: Incomplete | None = None) -> None: ...
    def __eq__(self, event): ...
    def __ne__(self, event): ...
    def is_sub_event(self, event): ...
    def is_event_or_sub_event(self, event): ...

def capability_and_super_capabilites(capability): ...
def is_capability_or_sub_capability(base_capability, capability): ...
def is_capability_or_sub_capability_in_list(base_capability, capability_list): ...
def convert_to_capability_list(capability_list): ...
def is_event_or_sub_event(base_event, event): ...
def is_event_or_super_event(base_event, event): ...
def is_event_or_sub_event_in_list(base_event, event_list): ...
def is_event_or_super_event_in_list(base_event, event_list): ...
def get_all_events_or_super_events_in_list(base_event, event_list): ...
def convert_to_event_list(event_list): ...

class PluginThread(Thread):
    plugin: Incomplete
    load_complete: bool
    end_load_complete: bool
    unload_complete: bool
    end_unload_complete: bool
    load_plugin_thread: Incomplete
    end_load_plugin_thread: Incomplete
    unload_plugin_thread: Incomplete
    end_unload_plugin_thread: Incomplete
    event_queue: Incomplete
    condition: Incomplete
    daemon: bool

    def __init__(self, plugin) -> None: ...
    def set_load_complete(self, value) -> None: ...
    def set_end_load_complete(self, value) -> None: ...
    def set_unload_complete(self, value) -> None: ...
    def set_end_unload_complete(self, value) -> None: ...
    def add_event(self, event) -> None: ...
    lazy_load_plugin_thread: Incomplete
    def process_event(self, event): ...
    def run(self) -> None: ...

class PluginEventThread(Thread):
    plugin: Incomplete
    method: Incomplete
    daemon: bool

    def __init__(self, plugin, method): ...
    def run(self) -> None: ...



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import logging

import colony

try:
    import unittest.mock as mock
except ImportError:
    mock = None


class LoggersTest(colony.ColonyTestCase):
    """
    Test case for the verification of logging related
    methods and functions of colony.
    """

    def test_memory_handler(self):
        memory_handler = colony.MemoryHandler()
        formatter = logging.Formatter("%(message)s")
        memory_handler.setFormatter(formatter)

        latest = memory_handler.get_latest()
        self.assertEqual(len(latest), 0)
        self.assertEqual(latest, [])

        record = logging.makeLogRecord(
            dict(msg="hello world", levelname=logging.getLevelName(logging.INFO))
        )
        memory_handler.emit(record)
        latest = memory_handler.get_latest()

        self.assertEqual(len(latest), 1)
        self.assertEqual(latest, ["hello world"])

        record = logging.makeLogRecord(
            dict(msg="hello world 2", levelname=logging.getLevelName(logging.ERROR))
        )
        memory_handler.emit(record)
        latest = memory_handler.get_latest()

        self.assertEqual(len(latest), 2)
        self.assertEqual(latest, ["hello world 2", "hello world"])

        latest = memory_handler.get_latest(level=logging.ERROR)

        self.assertEqual(len(latest), 1)
        self.assertEqual(latest, ["hello world 2"])

        latest = memory_handler.get_latest(level=logging.CRITICAL)

        self.assertEqual(len(latest), 0)
        self.assertEqual(latest, [])

        latest = memory_handler.get_latest(level=logging.INFO)

        self.assertEqual(len(latest), 2)
        self.assertEqual(latest, ["hello world 2", "hello world"])

        latest = memory_handler.get_latest(count=1, level=logging.INFO)

        self.assertEqual(len(latest), 1)
        self.assertEqual(latest, ["hello world 2"])

    def test_memory_handler_file(self):
        memory_handler = colony.MemoryHandler()
        formatter = logging.Formatter("%(message)s")
        memory_handler.setFormatter(formatter)

        latest = memory_handler.get_latest()
        self.assertEqual(len(latest), 0)
        self.assertEqual(latest, [])

        record = logging.makeLogRecord(
            dict(msg="hello world", levelname=logging.getLevelName(logging.INFO))
        )
        memory_handler.emit(record)
        record = logging.makeLogRecord(
            dict(msg="hello world 2", levelname=logging.getLevelName(logging.INFO))
        )
        memory_handler.emit(record)

        file = colony.legacy.BytesIO()

        memory_handler.flush_to_file(file, clear=False)

        file.seek(0)
        contents = file.read()

        self.assertEqual(contents, b"hello world\nhello world 2\n")

        file = colony.legacy.BytesIO()

        memory_handler.flush_to_file(file, reverse=False)

        file.seek(0)
        contents = file.read()

        self.assertEqual(contents, b"hello world 2\nhello world\n")

        latest = memory_handler.get_latest(count=1)
        self.assertEqual(len(latest), 0)

    def test_logstash_handler(self):
        if mock == None:
            self.skipTest("Skipping test: mock unavailable")

        mock_api_client = mock.Mock()
        mock_api_client_messages = []
        mock_api_client.log_bulk = (
            lambda messages, tag="default": mock_api_client_messages.extend(messages)
        )

        logstash_handler = colony.LogstashHandler(api=mock_api_client)
        formatter = logging.Formatter("%(levelname)s - %(message)s")
        logstash_handler.setFormatter(formatter)

        self.assertEqual(len(logstash_handler.messages), 0)

        record = logging.makeLogRecord(
            dict(msg="hello world", levelname=logging.getLevelName(logging.INFO))
        )
        logstash_handler.emit(record)
        self.assertEqual(len(logstash_handler.messages), 1)
        self.assertEqual(
            logstash_handler.messages[0]["message_fmt"], "INFO - hello world"
        )
        self.assertEqual(logstash_handler.messages[0]["message"], "hello world")
        self.assertEqual(logstash_handler.messages[0]["level"], "INFO")
        self.assertEqual(logstash_handler.messages[0]["logger"], None)
        self.assertEqual(logstash_handler.messages[0]["path"], "")

        logstash_handler.flush()
        self.assertEqual(len(logstash_handler.messages), 0)
        self.assertEqual(len(mock_api_client_messages), 1)
        self.assertEqual(
            mock_api_client_messages[0]["message_fmt"], "INFO - hello world"
        )
        self.assertEqual(mock_api_client_messages[0]["message"], "hello world")
        self.assertEqual(mock_api_client_messages[0]["level"], "INFO")
        self.assertEqual(mock_api_client_messages[0]["logger"], None)
        self.assertEqual(mock_api_client_messages[0]["path"], "")

        record = logging.makeLogRecord(
            dict(msg="hello world 2", levelname=logging.getLevelName(logging.INFO))
        )
        logstash_handler.emit(record)
        self.assertEqual(len(logstash_handler.messages), 1)
        self.assertEqual(
            logstash_handler.messages[0]["message_fmt"], "INFO - hello world 2"
        )
        self.assertEqual(logstash_handler.messages[0]["message"], "hello world 2")
        self.assertEqual(logstash_handler.messages[0]["level"], "INFO")
        self.assertEqual(logstash_handler.messages[0]["logger"], None)
        self.assertEqual(logstash_handler.messages[0]["path"], "")



from typing import Sequence, TypeVar

T = TypeVar("T")

def list_intersect(
    first_list: Sequence[T], second_list: Sequence[T]
) -> Sequence[T]: ...
def list_extend(
    base_list: Sequence[T], extension_list: Sequence[T], copy_base_list: bool = ...
) -> Sequence[T]: ...
def list_exclude(
    base_list: Sequence[T], exclusion_list: Sequence[T], copy_base_list: bool = ...
): ...
def list_no_duplicates(list: Sequence[T]) -> list[T]: ...



from typing import Literal, Mapping, Sequence

CodeSet = Literal["A", "B", "C"]

DIGIT_ENCODING_MAP: Mapping[str, str]
START_CODE_2_OF_5: str
END_CODE_2_OF_5: str
START_CODES_CODE_128: Mapping[str, int]
END_CODE_CODE_128: int

def encode_2_of_5(string_value: str) -> str: ...
def encode_code_128(value: str, code_set: CodeSet = ...) -> str: ...
def encode_code_39(value: str) -> str: ...
def _interleave_digits(first_digit: str, second_digit: str) -> str: ...
def _calculate_check_digit(character_values: Sequence[str]) -> int: ...
def _encode_code_set_a(string_value: str) -> Sequence[str]: ...
def _encode_code_set_b(string_value: str) -> Sequence[str]: ...
def _encode_code_set_c(string_value: str) -> Sequence[str]: ...
def _get_character_string(character_values: Sequence[str]) -> str: ...



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import os
import sys
import inspect
import functools
import itertools
import contextlib
import collections

import urllib  # @UnusedImport

ArgSpec = collections.namedtuple("ArgSpec", ["args", "varargs", "keywords", "defaults"])


@contextlib.contextmanager
def ctx_absolute():
    root = sys.path.pop(0)
    try:
        yield
    finally:
        sys.path.insert(0, root)


with ctx_absolute():
    try:
        import urllib2
    except ImportError:
        urllib2 = None

with ctx_absolute():
    try:
        import httplib
    except ImportError:
        httplib = None

with ctx_absolute():
    try:
        import http
    except ImportError:
        http = None

with ctx_absolute():
    try:
        import types
    except ImportError:
        types = None

with ctx_absolute():
    try:
        import urllib.error
    except ImportError:
        pass

with ctx_absolute():
    try:
        import urllib.request
    except ImportError:
        pass

with ctx_absolute():
    try:
        import http.client
    except ImportError:
        pass

with ctx_absolute():
    try:
        import importlib.util
    except ImportError:
        pass

try:
    import HTMLParser
except ImportError:
    import html.parser

    HTMLParser = html.parser

try:
    import cPickle
except ImportError:
    import pickle

    cPickle = pickle

try:
    import imp
except ImportError:
    import importlib

    imp = importlib

try:
    import importlib
except ImportError:
    import imp

    importlib = imp

try:
    import cStringIO
except ImportError:
    import io

    cStringIO = io

try:
    import StringIO as _StringIO
except ImportError:
    import io

    _StringIO = io

try:
    import urlparse as _urlparse
except ImportError:
    import urllib.parse

    _urlparse = urllib.parse

PYTHON_3 = sys.version_info[0] >= 3
""" Global variable that defines if the current Python
interpreter is at least Python 3 compliant, this is used
to take some of the conversion decision for runtime """

PYTHON_35 = sys.version_info[0] >= 3 and sys.version_info[1] >= 5
""" Global variable that defines if the current Python
interpreter is at least Python 3.5 compliant """

PYTHON_36 = sys.version_info[0] >= 3 and sys.version_info[1] >= 6
""" Global variable that defines if the current Python
interpreter is at least Python 3.6 compliant """

PYTHON_39 = sys.version_info[0] >= 3 and sys.version_info[1] >= 9
""" Global variable that defines if the current Python
interpreter is at least Python 3.9 compliant """

PYTHON_ASYNC = PYTHON_35
""" Global variable that defines if the current Python
interpreter support the async/await syntax responsible
for the easy to use async methods """

PYTHON_ASYNC_GEN = PYTHON_36
""" Global variable that defines if the current Python
interpreter support the async/await generator syntax
responsible for the async generator methods """

PYTHON_V = int("".join([str(v) for v in sys.version_info[:3]]))
""" The Python version integer describing the version of
a the interpreter as a set of three integer digits """

if PYTHON_3:
    LONG = int
else:
    LONG = long  # @UndefinedVariable

if PYTHON_3:
    BYTES = bytes
else:
    BYTES = str  # @UndefinedVariable

if PYTHON_3:
    UNICODE = str
else:
    UNICODE = unicode  # @UndefinedVariable

if PYTHON_3:
    OLD_UNICODE = None
else:
    OLD_UNICODE = unicode  # @UndefinedVariable

if PYTHON_3:
    STRINGS = (str,)
else:
    STRINGS = (str, unicode)  # @UndefinedVariable

if PYTHON_3:
    ALL_STRINGS = (bytes, str)
else:
    ALL_STRINGS = (bytes, str, unicode)  # @UndefinedVariable

if PYTHON_3:
    INTEGERS = (int,)
else:
    INTEGERS = (int, long)  # @UndefinedVariable

# saves a series of global symbols that are going to be
# used latter for some of the legacy operations
_ord = ord
_chr = chr
_str = str
_bytes = bytes
_range = range

try:
    _xrange = xrange  # @UndefinedVariable
except Exception:
    _xrange = None

if PYTHON_3:
    Request = urllib.request.Request
else:
    Request = urllib2.Request

if PYTHON_3:
    HTTPHandler = urllib.request.HTTPHandler
else:
    HTTPHandler = urllib2.HTTPHandler

if PYTHON_3:
    HTTPError = urllib.error.HTTPError
else:
    HTTPError = urllib2.HTTPError

if PYTHON_3:
    HTTPConnection = http.client.HTTPConnection  # @UndefinedVariable
else:
    HTTPConnection = httplib.HTTPConnection

if PYTHON_3:
    HTTPSConnection = http.client.HTTPSConnection  # @UndefinedVariable
else:
    HTTPSConnection = httplib.HTTPSConnection

try:
    _execfile = execfile  # @UndefinedVariable
except Exception:
    _execfile = None

try:
    _reduce = reduce  # @UndefinedVariable
except Exception:
    _reduce = None

try:
    _reload = reload  # @UndefinedVariable
except Exception:
    _reload = None

try:
    _unichr = unichr  # @UndefinedVariable
except Exception:
    _unichr = None


def with_meta(meta, *bases):
    return meta("Class", bases, {})


def eager(iterable):
    if PYTHON_3:
        return list(iterable)
    return iterable


def iteritems(associative):
    if PYTHON_3:
        return associative.items()
    return associative.iteritems()


def iterkeys(associative):
    if PYTHON_3:
        return associative.keys()
    return associative.iterkeys()


def itervalues(associative):
    if PYTHON_3:
        return associative.values()
    return associative.itervalues()


def items(associative):
    if PYTHON_3:
        return eager(associative.items())
    return associative.items()


def keys(associative):
    if PYTHON_3:
        return eager(associative.keys())
    return associative.keys()


def values(associative):
    if PYTHON_3:
        return eager(associative.values())
    return associative.values()


def xrange(start, stop=None, step=1):
    if PYTHON_3:
        return _range(start, stop, step) if stop else _range(start)
    return _xrange(start, stop, step) if stop else _range(start)


def range(start, stop=None, step=None):
    if PYTHON_3:
        return eager(_range(start, stop, step)) if stop else eager(_range(start))
    return _range(start, stop, step) if stop else _range(start)


def ord(value):
    if PYTHON_3 and type(value) == int:
        return value
    return _ord(value)


def chr(value):
    if PYTHON_3:
        return _bytes([value])
    if type(value) in INTEGERS:
        return _chr(value)
    return value


def chri(value):
    if PYTHON_3:
        return value
    if type(value) in INTEGERS:
        return _chr(value)
    return value


def bytes(value, encoding="latin-1", errors="strict", force=False):
    if not PYTHON_3 and not force:
        return value
    if value == None:
        return value
    if type(value) == _bytes:
        return value
    return value.encode(encoding, errors)


def str(value, encoding="latin-1", errors="strict", force=False):
    if not PYTHON_3 and not force:
        return value
    if value == None:
        return value
    if type(value) in STRINGS:
        return value
    return value.decode(encoding, errors)


def u(value, encoding="utf-8", errors="strict", force=False):
    if PYTHON_3 and not force:
        return value
    if value == None:
        return value
    if type(value) == UNICODE:
        return value
    return value.decode(encoding, errors)


def ascii(value, encoding="utf-8", errors="replace"):
    if is_bytes(value):
        value = value.decode(encoding, errors)
    else:
        value = UNICODE(value)
    value = value.encode("ascii", errors)
    value = str(value)
    return value


def orderable(value):
    if not PYTHON_3:
        return value
    return Orderable(value)


def is_str(value):
    return type(value) == _str


def is_unicode(value):
    if PYTHON_3:
        return type(value) == _str
    else:
        return type(value) == unicode  # @UndefinedVariable


def is_bytes(value):
    if PYTHON_3:
        return type(value) == _bytes
    else:
        return type(value) == _str  # @UndefinedVariable


def is_string(value, all=False):
    target = ALL_STRINGS if all else STRINGS
    return type(value) in target


def is_generator(value):
    if inspect.isgenerator(value):
        return True
    if type(value) in (itertools.chain,):
        return True
    if hasattr(value, "_is_generator"):
        return True
    return False


def is_async_generator(value):
    if not hasattr(inspect, "isasyncgen"):
        return False
    return inspect.isasyncgen(value)


def is_unittest(name="unittest"):
    current_stack = inspect.stack()
    for stack_frame in current_stack:
        for program_line in stack_frame[4]:
            is_unittest = not name in program_line
            if is_unittest:
                continue
            return True
    return False


def execfile(path, global_vars, local_vars=None, encoding="utf-8"):
    if local_vars == None:
        local_vars = global_vars
    if not PYTHON_3:
        return _execfile(path, global_vars, local_vars)
    file = open(path, "rb")
    try:
        data = file.read()
    finally:
        file.close()
    data = data.decode(encoding)
    code = compile(data, path, "exec")
    exec(code, global_vars, local_vars)  # @UndefinedVariable


def walk(path, visit, arg):
    for root, dirs, _files in os.walk(path):
        names = os.listdir(root)
        visit(arg, root, names)
        for dir in list(dirs):
            exists = dir in names
            not exists and dirs.remove(dir)


def getargspec(func):
    has_full = hasattr(inspect, "getfullargspec")
    if has_full:
        return ArgSpec(*inspect.getfullargspec(func)[:4])
    else:
        return inspect.getargspec(func)


def has_module(name):
    if PYTHON_3:
        try:
            spec = importlib.util.find_spec(name)
        except ImportError:
            return False
        if spec == None:
            return False
        return True
    try:
        file, _path, _description = imp.find_module(name)
    except ImportError:
        return False
    if file:
        file.close()
    return True


def new_module(name):
    if hasattr(types, "ModuleType"):
        return types.ModuleType(name)
    if hasattr(imp, "new_module"):
        return imp.new_module(name)
    raise ValueError("No module build method available")


def reduce(*args, **kwargs):
    if PYTHON_3:
        return functools.reduce(*args, **kwargs)
    return _reduce(*args, **kwargs)


def reload(*args, **kwargs):
    if PYTHON_3:
        return importlib.reload(*args, **kwargs)
    return _reload(*args, **kwargs)


def unichr(*args, **kwargs):
    if PYTHON_3:
        return _chr(*args, **kwargs)
    return _unichr(*args, **kwargs)


def urlopen(*args, **kwargs):
    if PYTHON_3:
        return urllib.request.urlopen(*args, **kwargs)
    else:
        return urllib2.urlopen(*args, **kwargs)  # @UndefinedVariable


def build_opener(*args, **kwargs):
    if PYTHON_3:
        return urllib.request.build_opener(*args, **kwargs)
    else:
        return urllib2.build_opener(*args, **kwargs)  # @UndefinedVariable


def urlparse(*args, **kwargs):
    return _urlparse.urlparse(*args, **kwargs)


def urlunparse(*args, **kwargs):
    return _urlparse.urlunparse(*args, **kwargs)


def parse_qs(*args, **kwargs):
    return _urlparse.parse_qs(*args, **kwargs)


def urlencode(*args, **kwargs):
    if PYTHON_3:
        return urllib.parse.urlencode(*args, **kwargs)
    else:
        return urllib.urlencode(*args, **kwargs)  # @UndefinedVariable


def quote(*args, **kwargs):
    if PYTHON_3:
        return urllib.parse.quote(*args, **kwargs)
    else:
        return urllib.quote(*args, **kwargs)  # @UndefinedVariable


def quote_plus(*args, **kwargs):
    if PYTHON_3:
        return urllib.parse.quote_plus(*args, **kwargs)
    else:
        return urllib.quote_plus(*args, **kwargs)  # @UndefinedVariable


def unquote(*args, **kwargs):
    if PYTHON_3:
        return urllib.parse.unquote(*args, **kwargs)
    else:
        return urllib.unquote(*args, **kwargs)  # @UndefinedVariable


def unquote_plus(*args, **kwargs):
    if PYTHON_3:
        return urllib.parse.unquote_plus(*args, **kwargs)
    else:
        return urllib.unquote_plus(*args, **kwargs)  # @UndefinedVariable


def cmp_to_key(*args, **kwargs):
    if PYTHON_3:
        return dict(key=functools.cmp_to_key(*args, **kwargs))  # @UndefinedVariable
    else:
        return dict(cmp=args[0])


def tobytes(self, *args, **kwargs):
    if PYTHON_3:
        return self.tobytes(*args, **kwargs)
    else:
        return self.tostring(*args, **kwargs)


def tostring(self, *args, **kwargs):
    if PYTHON_3:
        return self.tobytes(*args, **kwargs)
    else:
        return self.tostring(*args, **kwargs)


def StringIO(*args, **kwargs):
    if PYTHON_3:
        return cStringIO.StringIO(*args, **kwargs)
    else:
        return _StringIO.StringIO(*args, **kwargs)


def BytesIO(*args, **kwargs):
    if PYTHON_3:
        return cStringIO.BytesIO(*args, **kwargs)
    else:
        return cStringIO.StringIO(*args, **kwargs)


class Orderable(tuple):
    """
    Simple tuple type wrapper that provides a simple
    first element ordering, that is compatible with
    both the Python 2 and Python 3+ infra-structures.
    """

    def __cmp__(self, value):
        return self[0].__cmp__(value[0])

    def __lt__(self, value):
        return self[0].__lt__(value[0])



from os import PathLike
from typing import Any, Iterator, Mapping, Sequence, TypeVar

T = TypeVar("T")
K = TypeVar("K")
V = TypeVar("V")

FLOAT_PRECISION: int

class Decimal(float):
    places: int

    def __new__(cls, value: float = ...): ...

class JournaledList[T](list):
    def __init__(self, *args, **kwargs): ...
    def append(self, object: T): ...
    def remove(self, object: T): ...
    def clear_journal(self): ...
    def get_appends(self) -> Sequence[T]: ...
    def get_removes(self) -> Sequence[T]: ...

class OrderedMap[K, V]:
    tuples_list: Sequence[tuple[K, V]]

    def __init__(
        self, ordered_keys: bool = ..., map: Mapping[Any, Any] | None = ...
    ): ...
    def get(self, key: K, default_value: V | None = ...) -> V: ...
    def values(self) -> Sequence[V]: ...
    def items(self) -> Sequence[tuple[K, V]]: ...
    def extend(self, map: Mapping[K, V]): ...
    def keys(self) -> Sequence[K]: ...
    def itervalues(self) -> Iterator[V]: ...
    def iteritems(self) -> Iterator[tuple[K, V]]: ...
    def iterkeys(self) -> Iterator[K]: ...

class OrderedMapIterator[K, V]:
    ordered_map: OrderedMap[K, V]
    current_index: int

    def __init__(self, ordered_map: OrderedMap[K, V]): ...
    def next(self) -> V: ...

class MultipleValueMap[K, V]:
    def __init__(self): ...
    def get(
        self, key: K, default_value: V | Sequence[V] | None = ...
    ) -> V | Sequence[V]: ...
    def values(self) -> Sequence[V]: ...
    def items(self) -> Sequence[tuple[K, V]]: ...
    def keys(self) -> Sequence[K]: ...
    def itervalues(self) -> Iterator[V]: ...
    def iteritems(self) -> Iterator[tuple[K, V]]: ...
    def iterkeys(self) -> Iterator[K]: ...
    def unset(self, key: K, value: V): ...

class FormatTuple:
    format_string: str
    arguments: Sequence[Any]

    def __init__(self, format_string: str, *args): ...
    @staticmethod
    def build(format_string: str, *args) -> FormatTuple: ...
    def json_v(self, format: bool = ...): ...
    def format(self, format_string: str | None = ...): ...
    def get_format_string(self) -> str: ...
    def set_format_string(self, format_string: str): ...

class FileReference:
    path: PathLike[str]
    encoding: str | None

    def __init__(self, path: PathLike[str], encoding: str | None = ...): ...
    def read_all(self, mode: str = ...): ...

def is_dictionary(object: Any) -> bool: ...



from os import PathLike
from threading import Lock
from typing import BinaryIO, Callable, Sequence

PathTuple = tuple[str, PathLike[str], PathLike[str]]

PATH_TUPLE_PROCESS_METHOD_PREFIX: str
ADD_OPERATION: str
ADD_NO_REPLACE_OPERATION: str
REMOVE_OPERATION: str

class FileRotator:
    base_file_path: PathLike[str] | None
    maximum_file_size: int | None
    file_count: int | None
    current_file: BinaryIO | None
    current_file_size: int | None
    closed: bool

    def __init__(
        self,
        base_file_path: PathLike[str],
        maximum_file_size: int = ...,
        file_count: int = ...,
    ): ...
    def open(self): ...
    def close(self): ...
    def write(
        self, string_value: bytes | str, flush: bool = ..., encoding: str = ...
    ): ...
    def is_closed(self) -> bool: ...
    def _start_rotator(self): ...
    def _stop_rotator(self): ...
    def _open_current_file(self): ...
    def _close_current_file(self, rename: bool = ...): ...
    def _update_rotator(self): ...

class FileContext:
    def __init__(self): ...
    def resolve_file_path(self, file_path: PathLike[str]) -> PathLike[str]: ...
    def exists_file_path(self, file_path: PathLike[str]) -> bool: ...
    def is_directory_path(self, file_path: PathLike[str]) -> bool: ...
    def read_file(self, file_path: PathLike[str]) -> bytes: ...
    def write_file(self, file_path, file_contents: bytes): ...
    def remove_directory(
        self, directory_path: PathLike[str], handle_exception: bool = ...
    ): ...
    def remove_file(self, file_path: PathLike[str], handle_exception: bool = ...): ...
    def remove_directory_immediate(
        self, directory_path: PathLike[str], handle_exception: bool = ...
    ): ...
    def get_file_path(self, file_path: PathLike[str]) -> PathLike[str]: ...
    def _create_directory(self, file_path: PathLike[str]): ...

class TransactionContext:
    commit_callbacks_list: Sequence[Callable[[], None]]
    pre_commit_callbacks_list: Sequence[Callable[[], None]]
    rollback_callbacks_list: Sequence[Callable[[], None]]
    pre_rollback_callbacks_list: Sequence[Callable[[], None]]

    def __init__(self): ...
    def add_commit_callback(self, callback: Callable[[], None]): ...
    def remove_commit_callback(self, callback: Callable[[], None]): ...
    def add_pre_commit_callback(self, callback: Callable[[], None]): ...
    def remove_pre_commit_callback(self, callback: Callable[[], None]): ...
    def add_rollback_callback(self, callback: Callable[[], None]): ...
    def remove_rollback_callback(self, callback: Callable[[], None]): ...
    def add_pre_rollback_callback(self, callback: Callable[[], None]): ...
    def remove_pre_rollback_callback(self, callback: Callable[[], None]): ...
    def _call_commit_callbacks(self): ...
    def _call_pre_commit_callbacks(self): ...
    def _call_rollback_callbacks(self): ...
    def _call_pre_rollback_callbacks(self): ...

class FileImmediateContext(FileContext, TransactionContext):
    def __init__(self): ...
    def commit(self, remove_duplicates: bool = ...): ...
    def rollback(self): ...

class FileTransactionContext(FileContext, TransactionContext):
    transaction_level: int
    temporary_path: PathLike[str] | None
    path_tuples_list: list[PathTuple] | None
    access_lock: Lock | None

    def __init__(self, temporary_path: PathLike[str] | None = ...): ...
    def commit(self, remove_duplicates: bool = ...): ...
    def rollback(self): ...
    def _reset(self): ...
    def _cleanup(self): ...
    def _add_path_tuple(self, path_tuple: PathTuple): ...
    def _get_virtual_file_path(self, file_path: PathLike[str]): ...
    def _is_removed_file_path(self, file_path: PathLike[str]): ...
    def _process_path_tuple_add(self, path_tuple: PathTuple): ...
    def _process_path_tuple_add_no_replace(self, path_tuple: PathTuple): ...
    def _process_path_tuple_remove(self, path_tuple: PathTuple): ...



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import os
import setuptools

setuptools.setup(
    name="colony",
    version="1.4.22",
    author="Hive Solutions Lda.",
    author_email="development@hive.pt",
    description="Colony Framework",
    license="Apache License, Version 2.0",
    keywords="colony plugin framework web",
    url="http://getcolony.com",
    zip_safe=False,
    py_modules=["colony_adm", "colony_start", "colony_wsgi"],
    packages=[
        "colony",
        "colony.base",
        "colony.config",
        "colony.libs",
        "colony.test",
        "colony.test.base",
        "colony.test.libs",
    ],
    test_suite="colony.test",
    package_dir={"": os.path.normpath("src")},
    package_data={"colony": ["res/*", "**/*.pyi", "**/*.typed"]},
    entry_points={
        "console_scripts": [
            "cpm = colony_adm:main",
            "colony = colony_start:main",
            "colony_wsgi = colony_wsgi:main",
        ]
    },
    install_requires=["appier"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Utilities",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.0",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    long_description=open(os.path.join(os.path.dirname(__file__), "README.md"), "rb")
    .read()
    .decode("utf-8"),
    long_description_content_type="text/markdown",
)



from typing import Sequence

SIZE_UNITS_LIST: Sequence[str]
SIZE_UNIT_COEFFICIENT: int
DEFAULT_MINIMUM: int

def size_round_unit(
    size_value: int, minimum: int = ..., space: bool = ..., depth: int = ...
) -> str: ...



from typing import Any, Mapping, TypeVar

T = TypeVar("T")

def ceil_integer(value: int) -> int: ...
def greatest_common_divisor(p_value: int, q_value: int) -> int: ...
def fast_exponentiation(base: int, exponent: int, modulus: int) -> int: ...
def item_set_total(item_set: Mapping[Any, int]) -> int: ...
def item_set_percentage(item_name: T, item_set: Mapping[T, int]) -> float: ...
